/*
 * Whoson is a simple utility to produce a list of usernames 
 * currently logged in. It strives to remove duplicates from 
 * its output, printing only unique usernames.
 *
 * This was originally based on code posted to StackOverflow 
 * by user hugos in this post:
 * 	https://stackoverflow.com/questions/31472040
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/utsname.h>
#include <utmp.h>

struct node {
    char *username;
    struct node* next;
};
typedef struct node node_t;

// Function to create a new node on the heap
// returns a pointer to the node created
node_t *create_new_node(char *username) {
    node_t *result = malloc(sizeof(node_t));
    result->username = username;
    result->next = NULL;
    return result;
}

void printlist (node_t *head) {
  node_t *temp = head;

  while ( temp != NULL ) {
    printf("%s\n", temp->username);
    temp = temp->next;
  }
  return;
}


// Function to insert a node into the linked list
// that begins at "head". Returns a pointer to the head
node_t *insert_into_list(node_t *head, node_t *this) {
    node_t *temp = head;
    node_t *result = head;
    int int_res = 0;        // an integer result
    int insert_done = 0;    // 

    while (temp != NULL) {
      int_res = strcmp( this->username, temp->username );
      if (int_res < 0 ) {     // insert "this" node here
        if (temp == head) {   // special case for head node
          head = this;
        }
        this->next = temp->next;
        return result;
      } else {
        if (int_res == 0) { // we must already have this user
          return result;    // just bail and leave the list unchanged
        }
      }
      // Move on to the next node and retest the while
      temp = temp->next;    
    }

    return result;
}


int main(void)
{
  node_t *head;
  node_t *this;
  head = malloc(sizeof(node_t));

  struct utmp *ut_ent;
  setutent();     // 
  ut_ent=getutent();   // gets first utmp entry

  this = head;


  while(ut_ent) {
    if(ut_ent->ut_type==USER_PROCESS) {
      this->username = ut_ent->ut_user;
      head = insert_into_list(head, this);
    }
    ut_ent=getutent();
    this=malloc(sizeof(node_t));
  }
  printlist(head);
  return 0;
}
